@@ -1,21 +1,19 @@
-import datetime
-import warnings
-import re
-import time
 import os
+import re
+import warnings
+from datetime import datetime
+from datetime import timezone
+from typing import Callable
+from typing import Iterator
+from typing import List
+from typing import overload
+from typing import Tuple
 
 from .config import Configuration
+from .config import Version as PkgVersion
+from .utils import iter_entry_points
 from .utils import trace
 
-try:
-    from packaging.version import Version
-except ImportError:
-    import pkg_resources
-
-    Version = pkg_resources.packaging.version.Version
-
-
-from pkg_resources import iter_entry_points
 
 SEMVER_MINOR = 2
 SEMVER_PATCH = 3
@@ -23,7 +21,7 @@ SEMVER_LEN = 3
 
 
 def _parse_version_tag(tag, config):
-    tagstring = tag if not isinstance(tag, str) else str(tag)
+    tagstring = tag if isinstance(tag, str) else str(tag)
     match = config.tag_regex.match(tagstring)
 
     result = None
@@ -54,7 +52,7 @@ def callable_or_entrypoint(group, callable_or_name):
         return ep.load()
 
 
-def tag_to_version(tag, config=None):
+def tag_to_version(tag, config: "Configuration | None" = None):
     """
     take a tag that might be prefixed with a keyword and return only the version part
     :param config: optional configuration object
@@ -79,7 +77,7 @@ def tag_to_version(tag, config=None):
             )
         )
 
-    version = Version(version)
+    version = config.version_cls(version)
     trace("version", repr(version))
 
     return version
@@ -102,13 +100,13 @@ def tags_to_versions(tags, config=None):
 class ScmVersion:
     def __init__(
         self,
-        tag_version,
-        distance=None,
-        node=None,
-        dirty=False,
-        preformatted=False,
-        branch=None,
-        config=None,
+        tag_version: str,
+        distance: "int|None" = None,
+        node: "str|None" = None,
+        dirty: bool = False,
+        preformatted: bool = False,
+        branch: "str|None" = None,
+        config: "Configuration|None" = None,
         node_date=None,
         **kw,
     ):
@@ -120,9 +118,11 @@ class ScmVersion:
         self.distance = distance
         self.node = node
         self.node_date = node_date
-        self.time = datetime.datetime.utcfromtimestamp(
-            int(os.environ.get("SOURCE_DATE_EPOCH", time.time()))
-        )
+        if "SOURCE_DATE_EPOCH" in os.environ:
+            date_epoch = int(os.environ["SOURCE_DATE_EPOCH"])
+            self.time = datetime.fromtimestamp(date_epoch, timezone.utc)
+        else:
+            self.time = datetime.now(timezone.utc)
         self._extra = kw
         self.dirty = dirty
         self.preformatted = preformatted
@@ -167,24 +167,24 @@ class ScmVersion:
         return self.format_with(fmt, guessed=guessed)
 
 
-def _parse_tag(tag, preformatted, config):
+def _parse_tag(tag, preformatted, config: "Configuration|None"):
     if preformatted:
         return tag
-    if not isinstance(tag, Version):
+    if config is None or not isinstance(tag, config.version_cls):
         tag = tag_to_version(tag, config)
     return tag
 
 
 def meta(
     tag,
-    distance=None,
-    dirty=False,
-    node=None,
-    preformatted=False,
-    branch=None,
-    config=None,
+    distance: "int|None" = None,
+    dirty: bool = False,
+    node: "str|None" = None,
+    preformatted: bool = False,
+    branch: "str|None" = None,
+    config: "Configuration|None" = None,
     **kw,
-):
+) -> ScmVersion:
     if not config:
         warnings.warn(
             "meta invoked without explicit configuration,"
@@ -198,32 +198,45 @@ def meta(
     )
 
 
-def guess_next_version(tag_version):
+def guess_next_version(tag_version: ScmVersion):
     version = _strip_local(str(tag_version))
     return _bump_dev(version) or _bump_regex(version)
 
 
-def _strip_local(version_string):
+def _dont_guess_next_version(tag_version: ScmVersion):
+    version = _strip_local(str(tag_version))
+    return _bump_dev(version) or _add_post(version)
+
+
+def _strip_local(version_string: str) -> str:
     public, sep, local = version_string.partition("+")
     return public
 
 
-def _bump_dev(version):
+def _add_post(version: str):
+    if "post" in version:
+        raise ValueError(
+            f"{version} already is a post release, refusing to guess the update"
+        )
+    return f"{version}.post1"
+
+
+def _bump_dev(version: str) -> "str | None":
     if ".dev" not in version:
-        return
+        return None
 
     prefix, tail = version.rsplit(".dev", 1)
     if tail != "0":
         raise ValueError(
             "choosing custom numbers for the `.devX` distance "
             "is not supported.\n "
-            "The {version} can't be bumped\n"
-            "Please drop the tag or create a new supported one".format(version=version)
+            f"The {version} can't be bumped\n"
+            "Please drop the tag or create a new supported one ending in .dev0"
         )
     return prefix
 
 
-def _bump_regex(version):
+def _bump_regex(version: str) -> str:
     match = re.match(r"(.*?)(\d+)$", version)
     if match is None:
         raise ValueError(
@@ -235,14 +248,14 @@ def _bump_regex(version):
         return "%s%d" % (prefix, int(tail) + 1)
 
 
-def guess_next_dev_version(version):
+def guess_next_dev_version(version: ScmVersion):
     if version.exact:
         return version.format_with("{tag}")
     else:
         return version.format_next_version(guess_next_version)
 
 
-def guess_next_simple_semver(version, retain, increment=True):
+def guess_next_simple_semver(version: str, retain: int, increment=True):
     try:
         parts = [int(i) for i in str(version).split(".")[:retain]]
     except ValueError:
@@ -277,11 +290,15 @@ def release_branch_semver_version(version):
         # Does the branch name (stripped of namespace) parse as a version?
         branch_ver = _parse_version_tag(version.branch.split("/")[-1], version.config)
         if branch_ver is not None:
+            branch_ver = branch_ver["version"]
+            if branch_ver[0] == "v":
+                # Allow branches that start with 'v', similar to Version.
+                branch_ver = branch_ver[1:]
             # Does the branch version up to the minor part match the tag? If not it
             # might be like, an issue number or something and not a version number, so
             # we only want to use it if it matches.
             tag_ver_up_to_minor = str(version.tag).split(".")[:SEMVER_MINOR]
-            branch_ver_up_to_minor = branch_ver["version"].split(".")[:SEMVER_MINOR]
+            branch_ver_up_to_minor = branch_ver.split(".")[:SEMVER_MINOR]
             if branch_ver_up_to_minor == tag_ver_up_to_minor:
                 # We're in a release/maintenance branch, next is a patch/rc/beta bump:
                 return version.format_next_version(guess_next_version)
@@ -299,11 +316,11 @@ def release_branch_semver(version):
     return release_branch_semver_version(version)
 
 
-def no_guess_dev_version(version):
+def no_guess_dev_version(version: ScmVersion):
     if version.exact:
         return version.format_with("{tag}")
     else:
-        return version.format_with("{tag}.post1.dev{distance}")
+        return version.format_next_version(_dont_guess_next_version)
 
 
 def date_ver_match(ver):
@@ -317,7 +334,7 @@ def date_ver_match(ver):
     return match
 
 
-def guess_next_date_ver(version, node_date=None, date_fmt=None):
+def guess_next_date_ver(version, node_date=None, date_fmt=None, version_cls=None):
     """
     same-day -> patch +1
     other-day -> today
@@ -326,21 +343,28 @@ def guess_next_date_ver(version, node_date=None, date_fmt=None):
     """
     match = date_ver_match(version)
     if match is None:
-        raise ValueError(
-            "{version} does not correspond to a valid versioning date, "
-            "please correct or use a custom version scheme".format(version=version)
+        warnings.warn(
+            f"{version} does not correspond to a valid versioning date, "
+            "assuming legacy version"
         )
+        if date_fmt is None:
+            date_fmt = "%y.%m.%d"
+
     # deduct date format if not provided
     if date_fmt is None:
         date_fmt = "%Y.%m.%d" if len(match.group("year")) == 4 else "%y.%m.%d"
-    head_date = node_date or datetime.date.today()
+    today = datetime.now(timezone.utc).date()
+    head_date = node_date or today
     # compute patch
-    tag_date = datetime.datetime.strptime(match.group("date"), date_fmt).date()
+    if match is None:
+        tag_date = today
+    else:
+        tag_date = datetime.strptime(match.group("date"), date_fmt).date()
     if tag_date == head_date:
-        patch = match.group("patch") or "0"
+        patch = "0" if match is None else (match.group("patch") or "0")
         patch = int(patch) + 1
     else:
-        if tag_date > head_date:
+        if tag_date > head_date and match is not None:
             # warn on future times
             warnings.warn(
                 "your previous tag  ({}) is ahead your node date ({})".format(
@@ -352,8 +376,9 @@ def guess_next_date_ver(version, node_date=None, date_fmt=None):
         node_date=head_date, date_fmt=date_fmt, patch=patch
     )
     # rely on the Version object to ensure consistency (e.g. remove leading 0s)
-    # TODO: support for intentionally non-normalized date versions
-    next_version = str(Version(next_version))
+    if version_cls is None:
+        version_cls = PkgVersion
+    next_version = str(version_cls(next_version))
     return next_version
 
 
@@ -368,7 +393,11 @@ def calver_by_date(version):
             match = date_ver_match(ver)
             if match:
                 return ver
-    return version.format_next_version(guess_next_date_ver, node_date=version.node_date)
+    return version.format_next_version(
+        guess_next_date_ver,
+        node_date=version.node_date,
+        version_cls=version.config.version_cls,
+    )
 
 
 def _format_local_with_time(version, time_format):
@@ -412,7 +441,9 @@ def _get_ep(group, name):
         return ep.load()
 
 
-def _iter_version_schemes(entrypoint, scheme_value, _memo=None):
+def _iter_version_schemes(
+    entrypoint: str, scheme_value: "str|List[str]|Tuple[str, ...]", _memo=None
+) -> Iterator[Callable[["ScmVersion"], str]]:
     if _memo is None:
         _memo = set()
     if isinstance(scheme_value, str):
@@ -427,7 +458,23 @@ def _iter_version_schemes(entrypoint, scheme_value, _memo=None):
         yield scheme_value
 
 
-def _call_version_scheme(version, entypoint, given_value, default):
+@overload
+def _call_version_scheme(
+    version: ScmVersion, entypoint: str, given_value: str, default: str
+) -> str:
+    ...
+
+
+@overload
+def _call_version_scheme(
+    version: ScmVersion, entypoint: str, given_value: str, default: None
+) -> "str|None":
+    ...
+
+
+def _call_version_scheme(
+    version: ScmVersion, entypoint: str, given_value: str, default: "str|None"
+) -> "str|None":
     for scheme in _iter_version_schemes(entypoint, given_value):
         result = scheme(version)
         if result is not None:
@@ -435,7 +482,7 @@ def _call_version_scheme(version, entypoint, given_value, default):
     return default
 
 
-def format_version(version, **config):
+def format_version(version: ScmVersion, **config) -> str:
     trace("scm version", version)
     trace("config", config)
     if version.preformatted:
